<?php

class DB {
	
	// DBオブジェクト
	public $db;
	
	// DBディレクトリ
	private $dir;
	
	// テーブル名配列
	private $table;
	
	//---------------------------------------------------------
	//  コンストラクタ
	//---------------------------------------------------------
	
	public function __construct($dataDir, $workDir) {
		
		// ディレクトリ名を設定
		$this->dir = $dataDir;
		
		// テーブル名設定ファイルを解析
		$this->table = parse_ini_file($workDir . '/configs/table.ini');
		
	}
	
	//---------------------------------------------------------
	//  DBに接続
	//---------------------------------------------------------
	
	public function connect($file) {
		
		// SQLite3が使えない時は終了
		if (!class_exists('SQLite3')) {
			exit("Server Error: Class 'SQLite3' is not found.");
		}
		
		// ディレクトリに書き込み出来ない時は終了
		elseif (!is_writable($this->dir)) {
			exit("Directory Permission Error: '" . $this->dir . "' is not writable.");
		}
		
		try {
			
			// SQLite3で接続
			$this->db = new SQLite3($this->dir . '/' . $file);
			
			// タイムアウト時間を10秒に設定
			$this->db->busyTimeout(10000);
			
		}
		
		// 例外が発生した時
		catch (Exception $e) {
			exit('DB Connect Error: SQLite3 Driver is not found.');
		}
		
	}
	
	//---------------------------------------------------------
	//  DBに追加接続
	//---------------------------------------------------------
	
	public function attach($file, $name = 'at') {
		
		// DBファイル名を整形
		$atDB = $this->dir . '/' . $file;
		
		// クエリーを実行
		$this->query("attach '$atDB' as $name;");
		
	}
	
	//---------------------------------------------------------
	//  DB追加接続を切断
	//---------------------------------------------------------
	
	public function detach($name = 'at') {
		
		// クエリーを実行
		$this->query("detach $name;");
		
	}
	
	//---------------------------------------------------------
	//  クエリー実行
	//---------------------------------------------------------
	
	public function query($query) {
		
		// テスト用出力
		//echo '<br>' . $query . '<br>';
		
		// クエリーを実行
		try {
			return $this->db->query($query);
		}
		
		// 例外が発生した時
		catch (Exception $e) {
			return $this->exception($query);
		}
		
	}
	
	//---------------------------------------------------------
	//  レコードを取得
	//---------------------------------------------------------
	
	public function fetch($results) {
		
		// 1行レコードを取得
		return $results->fetchArray(SQLITE3_ASSOC);
		
	}
	
	//---------------------------------------------------------
	//  クエリー実行＆レコードを取得
	//---------------------------------------------------------
	
	public function queryFetch($query, $name = '') {
		
		// クエリーを実行
		$results = $this->query($query);
		
		// 1行レコードを取得
		$rec = $results->fetchArray(SQLITE3_ASSOC);
		
		// カラム指定がある時は一部だけ返す
		if ($rec && $name) {
			return $rec[$name];
		}
		
		// カラム指定がない時は全て返す
		else {
			return $rec;
		}
		
	}
	
	//---------------------------------------------------------
	//  クエリー実行（プリペーアドステートメント）
	//---------------------------------------------------------
	
	public function prepare($query, $params = '') {
		
		// クエリーを実行
		try {
			$stmt = $this->db->prepare($query);
		}
		
		// 例外が発生した時
		catch (Exception $e) {
			$stmt = $this->exception($query, true);
		}
		
		// パラメータが無い時は終了
		if (!$params) {
			return $stmt;
		}
		
		// パラメータを準備
		foreach ($params as $key => $val) {
			$stmt->bindParam(":$key", $$key);
			$$key = $val;
		}
		
		// SQLを実行
		$stmt->execute();
		
	}
	
	//---------------------------------------------------------
	//  DB接続を解除
	//---------------------------------------------------------
	
	public function close() {
		
		// 接続を解除
		$this->db->close();
		
	}
	
	//---------------------------------------------------------
	//  トランザクション開始  deferred  immediate  exclusive
	//---------------------------------------------------------
	
	public function begin($mode = 'exclusive') {
		
		// クエリーを実行
		$this->query("begin $mode;");
		
	}
	
	//---------------------------------------------------------
	//  コミット
	//---------------------------------------------------------
	
	public function commit() {
		
		// クエリーを実行
		$this->query('commit;');
		
	}
	
	//---------------------------------------------------------
	//  ロールバック
	//---------------------------------------------------------
	
	public function rollback() {
		
		// クエリーを実行
		$this->query('rollback;');
		
	}
	
	//---------------------------------------------------------
	//  SQL文のエスケープ
	//---------------------------------------------------------
	
	public function escape($str) {
		
		// 文字列をエスケープ
		return "'" . SQLite3::escapeString($str) . "'";
		
	}
	
	//---------------------------------------------------------
	//  SQLiteバージョンの取得
	//---------------------------------------------------------
	
	public function version() {
		
		// 配列で取得
		$versions = SQLite3::version();
		
		// バージョン文字列を返す
		return $versions['versionString'];
		
	}
	
	//---------------------------------------------------------
	//  DBファイルが存在するかチェック
	//---------------------------------------------------------
	
	public function exists($file) {
		
		// DBファイル名を整形
		$dbFile = $this->dir . '/' . $file;
		
		// DBファイルが存在しない時はfalseを返す
		if (!file_exists($dbFile) || !filesize($dbFile)) {
			return false;
		}
		
		// DBデータが存在する時はtrueを返す
		else {
			return true;
		}
		
	}
	
	//---------------------------------------------------------
	//  テーブル名を取得
	//---------------------------------------------------------
	
	public function table($name) {
		
		// テーブル名返す
		return $this->table[$name];
		
	}
	
	//---------------------------------------------------------
	//  全テーブル名の取得
	//---------------------------------------------------------
	
	public function tables($opt = false) {
		
		// 変数を初期化
		$tables = array();
		
		// クエリーを作成
		$query = "select name from sqlite_master where type = 'table';";
		
		// クエリーを実行
		$results = $this->query($query);
		
		// 実行結果を配列に格納
		while ($rec = $results->fetchArray(SQLITE3_ASSOC)) {
			$tables[] = current($rec);
		}
		
		// オプションがtrueの時はキーと値を反転
		if ($opt) {
			$tables = array_flip($tables);
		}
		
		// テーブルリストを返す
		return $tables;
		
	}
	
	//---------------------------------------------------------
	//  例外発生時の処理
	//---------------------------------------------------------
	
	public function exception($query, $prepare = false) {
		
		// リトライ回数を設定
		$retry = 10;
		
		// 待ち時間を設定
		$wait = 2;
		
		// リトライループ
		for ($i = 1; ; $i++) {
			
			// 指定秒数待つ
			sleep($wait);
			
			// クエリーを実行
			try {
				return (!$prepare) ? $this->db->query($query) : $this->db->prepare($query);
			}
			
			// 例外が発生した時
			catch (Exception $e) {
				
				// リトライ回数に達した時は終了
				if ($i === $retry) {
					exit;
				}
				
			}
			
		}
		
	}
	
}

